#!/usr/bin/perl

use strict;

use Carp;
use Getopt::Long;
use File::Basename;

BEGIN {
   select(STDERR); $| = 1; select(STDOUT); $| = 1;
}

my $options   = check_params();
 

my $filename = $options->{'file'};
my ($name, $path, $suffix) = fileparse($filename, qr/\.[^.]*/);
my $fn = $name . $suffix;



if ($options->{'out'}) {
	print "output file: $options->{'out'}\n";
	fastq2_to_fasta($filename, $options->{'out'});
}
else {
	my @filehandle_array = create_filehandles($fn, $path);
	print "output file: $filehandle_array[2]\n";
	fastq2_to_fasta($filehandle_array[1],$filehandle_array[2]);
}




sub create_filehandles {
	my ($fastq_filename,$dir) = @_;
	
	my @filename = split(/\./, $fastq_filename);
	my $fasta_filename = $filename[0].".fasta";
	
	my $fastq_filehandle = $dir.$fastq_filename;
	my $fasta_filehandle = $dir.$fasta_filename;
	
	return ($fasta_filename,$fastq_filehandle,$fasta_filehandle);
}

sub check_read {
	my ($fastq_filename) = @_;
	
	my @file = split(/\./,$fastq_filename);
	my $read_direction = substr("$file[0]", (length($file[0]) - 1));
	
	return $read_direction;
}

sub fastq2_to_fasta {
	my ($fastq_filehandle,$fasta_filehandle) = @_;
	
	my $line1;
	my $line2;
	my $line3;
	my $line4;
	my @read_fields1;
	my @id_fields;
	my $read_id;
	my @read_number;

	open my $fastq_fh, '<', $fastq_filehandle
	  or croak "Cannot open FASTQ file [$fastq_filehandle]: $!.\n";
	open my $fasta_fh, '>', $fasta_filehandle
	  or croak "Cannot open FASTA file [$fasta_filehandle]: $!.\n";
	
	
	while ($line1 = <$fastq_fh>) {
		$line2 = <$fastq_fh>;
		$line3 = <$fastq_fh>;
		$line4 = <$fastq_fh>;
	
		$line1=~s/^@/>/;
		print $fasta_fh $line1;
		print $fasta_fh $line2;
	}
	
	close $fastq_fh or croak "Failed to close FASTQ file [$fastq_filehandle]: $!.\n";
	close $fasta_fh or croak "Failed to close FASTA file [$fasta_filehandle]: $!.\n";
}

sub check_params {
	my @standard_options = ( "help+", "man+", "log_level:s" );
	my %options;

	GetOptions( \%options, @standard_options, "file:s", "out:s" );
	exec("pod2usage $0") if $options{'help'};
	exec("perldoc $0") if $options{'man'};
	exec("pod2usage $0") if ( !( $options{'file'} ) );
	return \%options;
}


__DATA__

=head1 NAME

   fastq_to_fasta.pl

=head1 COPYRIGHT

   Copyright (c) 2011 ACPFG, Universty of Queensland, Brisbane, QLD 4072, Australia.
   All rights reserved.
   Intended for ACPFG internal use only.

=head1 DESCRIPTION

   This script converts a fastq file to fasta file. The original sequence ID header will not be changed.
   Input:  File name
   Output: Creates fasta file in the same directory as the fastq input file 

=head1 SYNOPSIS

   fastq_to_fasta.pl -file <file name> [-out <outFile>] [-help] [-man] [-verbose] 

      -file  <file name>        Location of fastq file to be converted
      -out   <output file>      The output file name. If not specified, use input file name replacing to fasta extension.
      [-help]                   Displays basic usage information
      [-man]                    Displays more detailed information   
         
=cut
