#!/usr/bin/env python

# V0.2 output all in a file, output genotype info
# V0.1 initial version
# Author: Kenneth Chan


import os, sys
import argparse

try:
    custlib = os.path.join(os.path.dirname(__file__),'../lib')
except NameError:
    custlib = 'lib'
sys.path.append(custlib)

from mysqldbPool import mysqldbPool
from config import config


FILE_SIZE=50000

conn = mysqldbPool()


def get_genotype_info(locID):
  cultInfo = conn.statement("""SELECT T_Cult_Name_ID_Cult AS Cult_Name_ID, Cult_Allele
                               FROM T_Cults
                               WHERE T_SNP_Loc_ID = %s
                               ORDER BY Cults_ID""", (locID))
  return cultInfo


def get_cult_names():
  cultName = conn.statement("""SELECT ID_Cult, Cult_Name
                               FROM T_Cult_Name
                               ORDER BY ID_Cult""")
  return cultName


def get_snp_info():
  snp_infos = conn.statement("""SELECT Chromosome_ID AS Chr_ID,
                                       Reference_Name AS Ref_Name,
                                       Chromosome_Name AS Chr_Name,
                                       PREFIX,
                                       T_SNP_Name_ID_SNP AS SNP_ID,
                                CASE 
                                  WHEN T_SNP_Type_ID_Type=1 then 'P'
                                  WHEN T_SNP_Type_ID_Type=2 then 'S'
                                  WHEN T_SNP_Type_ID_Type=3 then 'R'
                                END AS SNP_Type, 
                                Chrom_Pos AS Chr_Pos,
                                Loc_ID
                                FROM `T_SNP`, `T_SNP_Name`, T_Chromosome, T_Reference
                                WHERE ID_SNP = T_SNP_Name_ID_SNP
                                  AND Chromosome_ID = T_Chromosome_Chromosome_ID
                                  AND Reference_ID = T_Reference_Reference_ID
                                ORDER BY SNP_ID, Chr_ID, T_SNP_Type_ID_Type, Chr_Pos""")
  return snp_infos



def export_snp_info(outDir):
  snp_infos = get_snp_info()
  last_snp_id = -1
  this_snpOutFH = None
  this_cultOutFH = None
  counter=0
  fCounter=1

  # create cultivar name array
  cult_names=get_cult_names()
  numOfCult = len(cult_names)


  for snp_info in snp_infos:
    # write the snp info to file
    # if this snp id is a new one, start a new line for this 
    if last_snp_id != snp_info['SNP_ID']:
      # if this reaches the required num of lines, open new files and reset things
      if (counter % FILE_SIZE == 0):
        if this_snpOutFH != None:
          this_snpOutFH.write("\n")
          this_snpOutFH.close()
          this_cultOutFH.close()
        this_snpOutFH = open(os.path.join(outDir,'snpInfo_%d.csv'%(fCounter)), 'w')
        this_snpOutFH.write("SNP_ID,SNP_Type,Ref_Name,Chr_Name,Chr_Pos,...repeat the last 4 header columns...")
        this_cultOutFH = open(os.path.join(outDir,'genotypeInfo_%d.csv'%(fCounter)), 'w')
        this_cultOutFH.write("SNP_ID")
        this_cultOutFH.write(",%s,%s,%s,%s" %('SNP_Type', 'Ref_Name', 'Chr_Name', 'Chr_Pos'))
        for name in cult_names:
          this_cultOutFH.write(",%s" %(name['Cult_Name']))
        this_cultOutFH.write("\n")
        counter = 0
        fCounter += 1
      this_snpOutFH.write("\n")
      this_snpOutFH.write("%s%0*d" %(snp_info['PREFIX'], config['snp_id_length'], snp_info['SNP_ID']))
      last_snp_id=snp_info['SNP_ID']
      counter += 1

    this_snpOutFH.write(",%s,%s,%s,%d" %(snp_info['SNP_Type'], snp_info['Ref_Name'], snp_info['Chr_Name'], snp_info['Chr_Pos']))


    # check if snp type is P, if yes, print its corresponding cultivar info
    if (snp_info['SNP_Type'] == 'P' or snp_info['SNP_Type'] == 'R'):
      this_cultOutFH.write("%s%0*d" %(snp_info['PREFIX'], config['snp_id_length'], snp_info['SNP_ID']))
      this_cultOutFH.write(",%s,%s,%s,%d" %(snp_info['SNP_Type'], snp_info['Ref_Name'], snp_info['Chr_Name'], snp_info['Chr_Pos']))

      genotype_infos = get_genotype_info(snp_info['Loc_ID'])
      if (len(genotype_infos) == numOfCult):
        for genotype_info in genotype_infos:
          this_cultOutFH.write(",%s" %(genotype_info['Cult_Allele']))
      else:
        # go through genotype_info based on cult_name sequence
        raise Exception('Not yet implemented that this snp only has a subset of the cultivars in the DB')
        
      this_cultOutFH.write("\n")

  this_snpOutFH.write("\n")
  this_snpOutFH.close()



if __name__ == '__main__':

  parser = argparse.ArgumentParser(description="Exporting SNP information in uniSNPdb")
  parser.add_argument('--outDir', nargs='?', required=True,
                   help='Output Directory')
  args = parser.parse_args()

  export_snp_info(args.outDir)

#END

