#!/usr/bin/env python

import os, sys, glob
import argparse

try:
    custlib = os.path.join(os.path.dirname(__file__),'../lib')
except NameError:
    custlib = 'lib'
sys.path.append(custlib)

from mysqldbPool import mysqldbPool
from queryMarkerDb import QueryMarkerDb
from upload_manager import Upload_manager
from config import config

def reverse_a_loaded_snp(chr_id):
  conn = mysqldbPool()
  last_prim_repeat_snp = conn.statement(""" 
    SELECT * 
    FROM `T_SNP` 
    WHERE `T_Chromosome_Chromosome_ID`=%s
      AND ( `T_SNP_Type_ID_Type`=1 OR `T_SNP_Type_ID_Type`=3 )
    ORDER BY `Input_Date` DESC, `Loc_ID` DESC
    LIMIT 1 """ %(chr_id))
  if last_prim_repeat_snp is None:
    return None
  thisSNPLocID = last_prim_repeat_snp[0]['Loc_ID']
  print("Delete thisSNPLocID: %d"%(thisSNPLocID))
  # delete cultivars for thisSNPLocID
  print("""DELETE FROM `T_Cults` WHERE `T_SNP_Loc_ID` = %s""" %(thisSNPLocID))
  conn.statement("""DELETE FROM `T_Cults` WHERE `T_SNP_Loc_ID` = %s""" %(thisSNPLocID))

  print("""DELETE FROM `T_SNP` WHERE `Loc_ID`=%s"""%(thisSNPLocID))
  conn.statement("""DELETE FROM `T_SNP` WHERE `Loc_ID`=%s"""%(thisSNPLocID))

  # Check if thisSNPID has other prim, prim-repeat type
  thisSNPID = last_prim_repeat_snp[0]['T_SNP_Name_ID_SNP']
  numOfThisSNPID = conn.statement("""
    SELECT COUNT(*)
    FROM `T_SNP`
    WHERE `T_SNP_Name_ID_SNP`=%d
    AND (`T_SNP_Type_ID_Type`=1
         OR `T_SNP_Type_ID_Type`=3)"""%(thisSNPID))
  # if no
  if numOfThisSNPID[0][0] == 0:
    # delete all 2nd snp for this snp id
    print("""DELETE FROM `T_SNP` WHERE `T_SNP_Name_ID_SNP`=%s AND `T_SNP_Type_ID_Type`=2"""%(thisSNPID))
    conn.statement("""DELETE FROM `T_SNP` WHERE `T_SNP_Name_ID_SNP`=%s AND `T_SNP_Type_ID_Type`=2"""%(thisSNPID))
    # delete snp_name
    print("""DELETE FROM `T_SNP_Name` WHERE `ID_SNP` = %s """%(thisSNPID))
    conn.statement("""DELETE FROM `T_SNP_Name` WHERE `ID_SNP` = %s """%(thisSNPID))

  return last_prim_repeat_snp[0]['Chrom_Pos']


def extract_resubmit_vcf(vcfFile, chrPos, outDir):
  # Create new out filename
  vcfFileSplit=os.path.splitext(os.path.basename(vcfFile))
  outFile = os.path.join(outDir, vcfFileSplit[0]+'_missed'+vcfFileSplit[1])
  # open vcfFile
  inFH = open(vcfFile, 'r')
  # open outFile
  outFH = open(outFile, 'w')
  startCp=0
  for aLine in inFH:
    if startCp:
      outFH.write(aLine)
      continue

    if aLine.startswith('#'):
      outFH.write(aLine)
    else:
      tabbed = aLine.split("\t")
      thisChromPos = int(tabbed[1])
      if thisChromPos == chrPos:
        outFH.write(aLine)
        startCp = 1


def get_vcf_job_info(thisJobID):
  jobInfo = QueryMarkerDb.get_job(thisJobID)
  if jobInfo is None:
    print("No jobID: %s"%(thisJobID))
    return None
  jobID = jobInfo[0]
  spID = jobInfo[1]
  refID = jobInfo[2]
  uploadTypeID = jobInfo[3]
  uploadSubdir=jobInfo[4]
  thisUploadDir = os.path.join(config['upload_dir'], uploadSubdir)
  if uploadTypeID == QueryMarkerDb.get_upload_type_id("vcf"):
    vcfFiles = glob.glob( os.path.join(thisUploadDir, "*_org.vcf"))
    thisChrName = Upload_manager.peepChrNameinVcf(vcfFiles[0])
    thisChrID = QueryMarkerDb.get_chrom_id(thisChrName, refID)
    return vcfFiles[0], thisChrID
  else:
    return None



if __name__ == '__main__':

  parser = argparse.ArgumentParser(description="Reverse the last loaded snp: delete the snp in database and extract the unloaded snp to a file for resubmission")
  parser.add_argument('--jobID', nargs='?', required=True, help='Job ID')
  parser.add_argument('--outDir', nargs='?', required=True, help='Output Directory')
  args = parser.parse_args()

  (this_vcf_file, thisChrID) = get_vcf_job_info(args.jobID)
  print("Org vcf file: %s"%(this_vcf_file))

  if thisChrID is None:
    print("Can not get Chromosome id from job id %s"%(args.jobID))
    exit()
  chr_pos = reverse_a_loaded_snp(thisChrID)
  extract_resubmit_vcf(this_vcf_file, chr_pos, args.outDir)

#END

